const cheerio = require('cheerio'); //Jquery for Node
const fs = require('fs');
const Table = require('cli-table');
const axios = require('axios');
const _ = require('lodash');
const Persist = require('./persist');

const Graph = require('./graph');

class Spider {
	constructor(obj = {}) {
		this.stats = {
			crawled: {
				success: 0,
				failure: 0
			},
			skipped: {
				pre: 0,
				post: 0
			},
			downloaded: 0
		};

		this.persistFile = new Persist(obj.persist);
		this.graph = new Graph(this.toStore, this.checkStore, this.persistFile);
	}

	persist() {
		console.log(this.graph.getVisited());
		return this.graph.hydrate();
	}

	toStore(pageUrl, params) {
		return { pageUrl, params };
	}

	checkStore(x, pageUrl, params) {
		return x.pageUrl && _.isEqual(params, x.params);
	}

	print() {
		let tableDesign = {
			head: ['Successful Crawls', 'Failed Crawls', 'Skipped Crawls', 'Downloaded', 'Total Visited' /*, 'Crawled Pages(from Beginning)'*/]
			// , colWidths: [100, 200]
		};

		//if (this.props.showStats) {
		let table = new Table(tableDesign);
		let { crawled, skipped } = this.stats;

		table.push([crawled.success, crawled.failure, skipped.post, this.stats.downloaded, this.graph.getVisited().length /*, this.state.visited.length*/]);
		console.log(table.toString());
		// }
	}

	crawl(pageUrl, params, method = "GET") {
		return new Promise((resolve, reject) => {

			if (this.graph.isVisited(pageUrl, params)) {
				this.stats.skipped.post++;
				this.print();
				reject("Skipped");
			} else {
				axios({
					url: pageUrl,
					method: "GET",
					params: params
				}).then(data => {
					this.stats.crawled.success++;
					this.print();
					resolve(cheerio.load(data.data));
				}).catch(err => {
					this.stats.crawled.failure++;
					this.print();
					reject(err);
				});
				this.graph.setVisit(pageUrl, params);
			}
		});
	}

}

module.exports = Spider;